#!/bin/bash

# Find out where this file is located:
THIS_FILE="${BASH_SOURCE[0]}"
while [ -h "$THIS_FILE" ]; do # resolve $THIS_FILE until the file is no longer a symlink
  DIR="$( cd -P "$( dirname "$THIS_FILE" )" && pwd )"
  THIS_FILE="$(readlink "$THIS_FILE")"
  [[ $THIS_FILE != /* ]] && THIS_FILE="$DIR/$THIS_FILE" # if $THIS_FILE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
done

set -e
source $(dirname $THIS_FILE)/test-source_cmb_envvars.sh

NUM_WORKERS=3
NAME="CMB-Flask"
mkdir -p $CMB_LOG_FOLDER
GUNICORN_LOG="$CMB_LOG_FOLDER/gunicorn.log"

# Necessary due to the time it takes for checker.cpp to compile and check big test
# results on shortest path timing out the default timeout of gunicorn workers
GUNICORN_WORKER_TIMEOUT=120

echo "Starting $NAME"

# Start unicorn
GUNICORN_CONFIG=$FLASK_DIR/configs/test-gunicorn.cfg
if [[ ! -f $GUNICORN_CONFIG ]]; then
  $FLASK_VENV_DIR/bin/gunicorn manager:app -b $CMB_SERVER:$SERVER_PORT \
    --preload \
    --name $NAME \
    --workers $NUM_WORKERS \
    --chdir=$FLASK_DIR/source \
    --timeout $GUNICORN_WORKER_TIMEOUT \
    --user=$CMB_USER --group=$CMB_USER_GROUP \
    --log-syslog-prefix="$NAME Gunicorn" \
    --log-level=$GUNICORN_LOG_LEVEL \
    --access-logfile $GUNICORN_LOG \
    --log-file=-
else
  # Not implemented due to file not existing
  exit 1
fi
